! Simple program for the propagation of the circular restricted three-body problem (planar)
! course of Elementi di Meccanica Celeste (UniPi)
program cr3bpEMC

  use hamiltonian
  use integr

  implicit none

  character(256):: hamParamFile ! namelist containing the parameters of the Hamiltonian function
  character(256):: intParamFile ! namelist containing the parameters of the numerical integration
  character(256):: outFile      ! output file

  type(hamparam):: Hp  ! parameters of the Hamiltonian
  type(intparam):: Ip  ! parameters of the numerical integration

  double precision:: X(4) ! state vector
  double precision:: t ! time
  integer:: Nstep, Nreg

!---------------------------------------------------------------------------------------------
! Reading and computing the parameters

  if(COMMAND_ARGUMENT_COUNT() >= 3) then
     call GET_COMMAND_ARGUMENT(1,hamParamFile)
     call GET_COMMAND_ARGUMENT(2,intParamFile)
     call GET_COMMAND_ARGUMENT(3,outFile)
  else
     write(*,*) "Use: <program name> <parameterFile.nml> <initialConditions.nml> <outFile.dat>"
     stop
  endIf

  ! computing the parameters of the Hamiltonian
  call readHamParam(hamParamFile,trim(outFile)//"_ham.nml",Hp)

  ! computing the parameters of the numerical integration
  call initInteg(intParamFile,trim(outFile)//"_int.nml", Hp,Ip, X)

!---------------------------------------------------------------------------------------------
! Initializations

  t = Ip%t0
  Nstep = 0 ! number of timesteps realised

  ! Opening and writing the header of the output file
  open(newunit=Nreg,file=trim(outFile),status='REPLACE',action='WRITE') ! opening the output file
  write(Nreg,'(A,4(I0,A))') "# t ; x, y, vx, vy;"

!---------------------------------------------------------------------------------------------
! Numerical integration

  call printFunc(X,t) ! saving the initial state
  call rkg(X, t, force, Ip%Dt, Ip%tSpan, printFunc) ! numerical integration

!---------------------------------------------------------------------------------------------

  close(Nreg) ! closing the output file

!---------------------------------------------------------------------------------------------
contains
!---------------------------------------------------------------------------------------------

  ! Force function for the numerical integration
  ! 
  function force(XX,tt)
     double precision,intent(in):: XX(:), tt
     double precision:: force(size(XX))

     ! NOTE: Kepler problem does not depend explicitely from the time tt

     ! Conservative part of the force.
     force(1:4) = DHam(Hp,XX(1:4))

  end function force

!---------------------------------------------------------------------------------------------

  ! Subroutine used after each step for saving the results.
  ! 
  subroutine printFunc(XX,tt)
     double precision,intent(in):: XX(:), tt
     double precision:: eps,y(2),r1,r2,J

     if( mod(Nstep,Ip%Nsave) == 0 ) then

        ! compute Jacobi constant
        y(1) = XX(1) + Hp%mu
        y(2) = XX(2)
        r1 = sqrt(DOT_PRODUCT(y(1:2),y(1:2)))
        y(1) = XX(1) - (1.d0 - Hp%mu)
        y(2) = XX(2)
        r2 = sqrt(DOT_PRODUCT(y(1:2),y(1:2)))
        J = 0.5d0*(XX(3)**2+XX(4)**2) - 0.5d0*(XX(1)**2+XX(2)**2) - (1.d0-Hp%mu)/r1 - Hp%mu/r2 
!print*, Hp%mu1,Hp%mu2,Hp%nm,J,eps,Hp%d
!stop  
   
        write(Nreg,'(ES22.15,*(1X,ES22.15))') tt, XX, J

     endIf
     Nstep = Nstep + 1

  end subroutine printFunc

!---------------------------------------------------------------------------------------------

end program cr3bpEMC
